@@ -1,5 +1,3 @@
-#!/usr/bin/env python3
-# -*- coding: utf-8 -*-
 R"""
     ----------------------------------------------------------
             __     __  High Octane Triage Analysis          __
@@ -34,7 +32,9 @@ various `refinery.units.Unit`s can be combined.
 4. `refinery.units`: writing custom units, add command-line arguments, and how to use refinery
    units within Python code.
 """
-__version__ = '0.8.28'
+from __future__ import annotations
+
+__version__ = '0.9.2'
 __distribution__ = 'binary-refinery'
 
 from typing import cast, Dict, List, Optional, Type, TypeVar, Iterable, TYPE_CHECKING
@@ -123,7 +123,7 @@ class __unit_loader__:
             self.reloading = True
             self.clear()
             for executable in get_all_entry_points():
-                name = executable.__qualname__
+                name = executable.__name__
                 self.units[name] = executable.__module__
                 self.cache[name] = executable
             self.save()
@@ -181,9 +181,13 @@ class __pdoc__(dict):
                     for method in abstractmethods:
                         if method.startswith('_'):
                             continue
-                        at = getattr(unit, method, None)
+                        at = getattr(unit, method, NotImplemented)
                         bt = getattr(unit.mro()[1], method, None)
-                        if at and at is not bt:
+                        if at is NotImplemented:
+                            continue
+                        if at is None:
+                            continue
+                        if at is not bt:
                             self[F'{name}.{method}'] = False
                 if hlp := get_help_string(unit, width=97):
                     hlp = hlp.replace('\x60', '')
