@@ -1,8 +1,8 @@
-#!/usr/bin/env python3
-# -*- coding: utf-8 -*-
 """
 https://en.wikipedia.org/wiki/List_of_CIL_instructions
 """
+from __future__ import annotations
+
 from refinery.lib.dotnet.disassembler.model import (
     CallSiteDescr,
     Class,
