@@ -1,5 +1,3 @@
-#!/usr/bin/env python3
-# -*- coding: utf-8 -*-
 """
 This module implements an abstraction layer executable loader for PE, ELF, and MachO files.
 The provided interface is the same for all executables. It powers the following units:
@@ -13,22 +11,17 @@ from __future__ import annotations
 
 import sys
 import itertools
+import codecs
 
 from typing import NamedTuple, TYPE_CHECKING
 
 from abc import ABC, abstractmethod
 from enum import Enum
-from uuid import uuid4, UUID
 from functools import lru_cache
 
 from refinery.lib import lief
 from refinery.lib.types import INF, ByteStr
-from refinery.lib.exceptions import MissingModule
-
-try:
-    import capstone as cs
-except ImportError:
-    cs = MissingModule('capstone')
+from refinery.lib.shared import capstone as cs
 
 if TYPE_CHECKING:
     from lief.ELF import Binary as ELFBinary
@@ -44,14 +37,12 @@ if TYPE_CHECKING:
         Iterable,
         List,
         Optional,
-        ParamSpec,
         Type,
         TypeVar,
         Union,
     )
 
     _T = TypeVar('_T')
-    _P = ParamSpec('_P')
 
     AnyLIEF = Union[
         MachOBinary,
@@ -107,6 +98,11 @@ class Range(NamedTuple):
         """
         return slice(self.lower, self.upper)
 
+    def __eq__(self, other):
+        if not isinstance(other, Range):
+            return False
+        return other.lower == self.lower and other.upper == self.upper
+
     def __len__(self):
         return self.upper - self.lower
 
@@ -164,29 +160,25 @@ class ArchItem(NamedTuple):
     An item of the `refinery.lib.executable.Arch` enumeration. It is used to store the register
     size in bits for a given architecture.
     """
-    id: UUID
     pointer_size: int
-
-    @classmethod
-    def New(cls, pointer_size: int):
-        return cls(uuid4(), pointer_size)
+    id: str
 
 
 class Arch(ArchItem, Enum):
     """
     An enumeration of supported architectures and their register sizes.
     """
-    X32 = ArchItem.New(32)
-    X64 = ArchItem.New(64)
-    ARM32 = ArchItem.New(32)
-    ARM64 = ArchItem.New(64)
-    MIPS16 = ArchItem.New(16)
-    MIPS32 = ArchItem.New(32)
-    MIPS64 = ArchItem.New(64)
-    PPC32 = ArchItem.New(32)
-    PPC64 = ArchItem.New(64)
-    SPARC32 = ArchItem.New(32)
-    SPARC64 = ArchItem.New(64)
+    X32 = (32, 'X32')
+    X64 = (64, 'X64')
+    ARM32 = (32, 'ARM32')
+    ARM64 = (64, 'ARM64')
+    MIPS16 = (16, 'MIPS16')
+    MIPS32 = (32, 'MIPS32')
+    MIPS64 = (64, 'MIPS64')
+    PPC32 = (32, 'PPC32')
+    PPC64 = (64, 'PPC64')
+    SPARC32 = (32, 'SPARC32')
+    SPARC64 = (64, 'SPARC64')
 
 
 class LT(str, Enum):
@@ -228,6 +220,11 @@ class Section(NamedTuple):
         sections = [self] if populate_sections else None
         return Segment(self.physical, self.virtual, sections, self.name)
 
+    def __eq__(self, other):
+        if not isinstance(other, Section):
+            return False
+        return other.physical == self.physical and other.virtual == self.virtual
+
     def __str__(self):
         return str(self.as_segment())
 
@@ -235,16 +232,25 @@ class Section(NamedTuple):
         return F'<{self.__class__.__name__}:{self!s}>'
 
 
+class Relocation(NamedTuple):
+    """
+    The data required to apply a relocation; An integer of the given size at the given address has
+    to be patched to the given value.
+    """
+    address: int
+    value: int
+    size: int
+
+
 class Symbol(NamedTuple):
     address: int
     name: Optional[str] = None
-    code: bool = True
-    exported: bool = True
-    is_entry: bool = False
     size: Optional[int] = None
-    tls_index: Optional[int] = None
-    type_name: Optional[str] = None
-    bind_name: Optional[str] = None
+    function: Optional[bool] = None
+    exported: Optional[bool] = None
+    imported: Optional[bool] = None
+    is_entry: bool = False
+    section: Optional[Section] = None
 
     def get_name(self):
         name = self.name
@@ -252,7 +258,7 @@ class Symbol(NamedTuple):
             return name
         if self.is_entry:
             return 'entry'
-        if self.code:
+        if self.function:
             return F'sub_{self.address:08X}'
         else:
             return F'sym_{self.address:08X}'
@@ -272,6 +278,11 @@ class Segment(NamedTuple):
             raise ValueError('Unable to convert nameless segment to section.')
         return Section(self.name, self.physical, self.virtual, False)
 
+    def __eq__(self, other):
+        if not isinstance(other, Segment):
+            return False
+        return other.virtual == self.virtual and other.physical == self.physical
+
     def __str__(self):
         msg = F'P=[{self.physical!s}];V=[{self.virtual!s}]'
         if self.name is not None:
@@ -349,6 +360,14 @@ class Executable(ABC):
         else:
             return True
 
+    def read_integer(self, address: int, size: Optional[int] = None):
+        """
+        Read an integer at the given address. The size defaults to the pointer size in bytes.
+        """
+        if size is None:
+            size = self.pointer_size_in_bytes
+        return int.from_bytes(self[address:address + size], self.byte_order().value)
+
     def read(self, key: Union[int, slice, Range]) -> memoryview:
         """
         Read data from the binary based on a given address. If the input `key` is a single integer,
@@ -387,7 +406,7 @@ class Executable(ABC):
             if b == 0:
                 string = string[:k]
                 break
-        return string.decode('latin-1')
+        return codecs.decode(string, 'latin-1')
 
     def rebase_usr_to_img(self, addr: int) -> int:
         return addr - self.base + self.image_defined_base()
@@ -426,6 +445,13 @@ class Executable(ABC):
         """
         return self.arch().pointer_size
 
+    @property
+    def pointer_size_in_bytes(self) -> int:
+        """
+        Return the size of a pointer in bytes. Depends on `refinery.lib.executable.Executable.arch`.
+        """
+        return self.pointer_size // 8
+
     def location_from_address(self, address: int) -> Location:
         """
         Return a `refinery.lib.executable.Location` from the given address.
@@ -538,12 +564,22 @@ class Executable(ABC):
     def _segments(self, populate_sections=False) -> Generator[Segment, None, None]:
         ...
 
+    @abstractmethod
+    def _relocations(self) -> Generator[Relocation, None, None]:
+        ...
+
     def segments(self, populate_sections=False) -> Generator[Segment, None, None]:
         """
         An iterable of all `refinery.lib.executable.Segment`s in this executable.
         """
         yield from self._segments(populate_sections=populate_sections)
 
+    def relocations(self) -> Generator[Relocation, None, None]:
+        """
+        An iterable of all `refinery.lib.executable.Relocation`s defined in this executable.
+        """
+        yield from self._relocations()
+
     def sections(self) -> Generator[Section, None, None]:
         """
         An iterable of all `refinery.lib.executable.Section`s in this executable.
@@ -640,6 +676,9 @@ class ExecutableCodeBlob(Executable):
     def _symbols(self) -> Generator[Symbol, None, None]:
         yield Symbol(0, is_entry=True)
 
+    def _relocations(self):
+        yield from ()
+
     def _sections(self) -> Generator[Section, None, None]:
         v = Range(self.base, self.base + len(self.data))
         p = Range(0, len(self.data))
@@ -722,11 +761,84 @@ class LIEF(Executable):
                 return Arch.X64
         raise NotImplementedError
 
+    def _relocations(self):
+        ps = self.pointer_size_in_bytes
+        it: Iterable[lief.Relocation] = iter(self._lh.relocations)
+
+        if self.is_elf:
+            for rel in it:
+                assert isinstance(rel, lief.ELF.Relocation)
+                if (symbol := rel.symbol) is None:
+                    continue
+                if '_RELATIVE' in (tn := rel.type.name):
+                    base = self.base
+                elif tn.endswith('GLOB_DAT') or tn.endswith('JUMP_SLOT'):
+                    base = symbol.value
+                else:
+                    continue
+                yield Relocation(rel.address, base + rel.addend, ps)
+        elif (delta := (base := self.base) - self.image_defined_base()) != 0:
+            def relocation(addr: int, size: int):
+                return Relocation(addr, self.read_integer(addr, size) + delta & mask, size)
+            mask = (1 << self.pointer_size) - 1
+            for rel in it:
+                if isinstance(rel, lief.MachO.Relocation):
+                    yield relocation(rel.address, rel.size // 8)
+                elif isinstance(rel, lief.PE.Relocation):
+                    rva = rel.virtual_address
+                    if (delta := (base := self.base) - self.image_defined_base()) == 0:
+                        continue
+                    for entry in rel.entries:
+                        if entry.type == lief.PE.RelocationEntry.BASE_TYPES.HIGHLOW:
+                            size = 4
+                        elif entry.type == lief.PE.RelocationEntry.BASE_TYPES.DIR64:
+                            size = 8
+                        else:
+                            continue
+                        if size != entry.size:
+                            raise RuntimeError(F'Unexpected relocation size: Guessed {size}, LIEF says {entry.size}.')
+                        yield relocation(base + rva + entry.position, size)
+                else:
+                    raise TypeError(F'Unexpected relocation type: {type(rel).__qualname__}')
+
     def _symbols(self) -> Generator[Symbol, None, None]:
         yield Symbol(self._lh.entrypoint, is_entry=True)
         it: Iterable[lief.Symbol] = self._lh.symbols
         for symbol in it:
-            yield Symbol(symbol.value, symbol.name, size=symbol.size)
+            addr = symbol.value
+            name = self.ascii(symbol.name)
+            size = symbol.size
+            if isinstance(symbol, lief.PE.Symbol):
+                yield Symbol(
+                    addr, name, size,
+                    function=symbol.complex_type == lief.PE.SYMBOL_COMPLEX_TYPES.FUNCTION,
+                    section=self._convert_section(s) if (s := symbol.section) else None,
+                )
+            elif isinstance(symbol, lief.PE.ImportEntry):
+                yield Symbol(
+                    addr, name, size,
+                    function=True,
+                    imported=True,
+                    exported=False,
+                )
+            elif isinstance(symbol, lief.ELF.Symbol):
+                name = self.ascii(symbol.demangled_name) or name
+                yield Symbol(
+                    addr, name, size,
+                    exported=symbol.exported,
+                    imported=symbol.imported,
+                    function=symbol.is_function,
+                    section=self._convert_section(s) if (s := symbol.section) else None,
+                )
+            elif isinstance(symbol, lief.MachO.Symbol):
+                name = self.ascii(symbol.demangled_name) or name
+                yield Symbol(
+                    addr, name, size,
+                    imported=symbol.is_external,
+                    exported=symbol.has_export_info,
+                )
+            else:
+                yield Symbol(addr, name, size)
 
     def _convert_section(self, section: lief.Section, segment_name: Optional[str] = None) -> Section:
         p_lower = section.offset
@@ -779,7 +891,8 @@ class LIEF(Executable):
                 yield segment.as_section()
             if self.is_pe:
                 return
-            yield from segment.sections
+            if sections := segment.sections:
+                yield from sections
 
     def _segments(self, populate_sections=False) -> Generator[Segment, None, None]:
         if self.is_pe:
