@@ -1,5 +1,3 @@
-#!/usr/bin/env python3
-# -*- coding: utf-8 -*-
 """
 An emulator for Inno Setup executables. The implementation is unlikely to be 100% correct as it
 was engineered by making various malicious scripts execute reasonably well, not by implementing
@@ -31,14 +29,13 @@ from typing import (
 
 from dataclasses import dataclass, field
 from enum import auto, Enum, IntFlag
-from functools import partial, wraps
+from functools import partial, wraps, cached_property
 from pathlib import Path
 from string import Formatter
 from time import process_time
 from urllib.parse import unquote
 from datetime import datetime, timedelta
 
-from refinery.lib.tools import cached_property
 from refinery.lib.types import CaseInsensitiveDict
 from refinery.lib.inno.archive import InnoArchive, Flags
 from refinery.lib.types import AST, INF, NoMask
