@@ -1,8 +1,8 @@
-#!/usr/bin/env python3
-# -*- coding: utf-8 -*-
 """
 File type related functions.
 """
+from __future__ import annotations
+
 import functools
 
 from refinery.lib.magic import magic, magicparse
