@@ -1,16 +1,14 @@
-#!/usr/bin/env python3
-# -*- coding: utf-8 -*-
 """
 Library of regular expression patterns.
 """
+from __future__ import annotations
+
 import enum
 import functools
 import re
 
-from typing import Optional
-
 from refinery.lib.patterns.tlds import tlds
-from refinery.lib.tools import cached_property, normalize_to_identifier, normalize_to_display
+from refinery.lib.tools import normalize_to_identifier, normalize_to_display
 
 
 class pattern:
@@ -20,9 +18,7 @@ class pattern:
     operators.
     """
     str_pattern: str
-    bin_pattern: Optional[bytes]
-    bin_compiled: re.Pattern
-    str_compiled: re.Pattern
+    bin_pattern: bytes | None
 
     def __init__(self, pattern: str, flags: int = 0):
         self.str_pattern = pattern
@@ -32,12 +28,12 @@ class pattern:
     def __bytes__(self):
         return self.bin_pattern
 
-    @cached_property
-    def bin_compiled(self):
+    @functools.cached_property
+    def bin(self):
         return re.compile(B'(%s)' % self.bin_pattern, flags=self.regex_flags)
 
-    @cached_property
-    def str_compiled(self):
+    @functools.cached_property
+    def str(self):
         return re.compile(self.str_pattern, flags=self.regex_flags)
 
     def __str__(self):
@@ -46,10 +42,10 @@ class pattern:
     def __getattr__(self, verb):
         if not hasattr(re.Pattern, verb):
             raise AttributeError(verb)
-        bin_attr = getattr(self.bin_compiled, verb)
+        bin_attr = getattr(self.bin, verb)
         if not callable(bin_attr):
             return bin_attr
-        str_attr = getattr(self.str_compiled, verb)
+        str_attr = getattr(self.str, verb)
 
         def wrapper(*args, **kwargs):
             for argument in args:
@@ -203,6 +199,15 @@ _pattern_number = (
 
 
 _pattern_date_elements = {
+    'A': '(?:{})'.format('|'.join([
+        '[sS]un(?:day)?',
+        '[mM]on(?:day)?',
+        '[tT]ue(?:sday)?',
+        '[wW]ed(?:nesday)?',
+        '[tT]hu(?:rsday)?',
+        '[fF]ri(?:day)?',
+        '[sS]at(?:urday)?',
+    ])),
     'B': '(?:{})'.format('|'.join([
         '[jJ]an(?:uary)?',
         '[fF]eb(?:ruary)?',
@@ -229,12 +234,14 @@ _pattern_date_elements = {
     'y': '(?:[0-9]{2})',
     'Y': '(?:[0-9]{4})',
     'c': '(?:[,;]|\\s|[,;]\\s)',
+    'gap': '\\s{1,3}'
 }
 
 _pattern_time = r'(?:{H}:{M}(?::{S})?|{I}:{M}(?::{S})?{c}?\(?{p}\)?)'.format_map(_pattern_date_elements)
 _pattern_date_elements['T'] = _pattern_time
 
 _pattern_date_list = [
+    R'{A}{c}(?:{d}|{D}){gap}{B}{c}{Y}(?:\s{T})?',
     R'{B}\s(?:{d}|{D}){c}{Y}(?:\s{T})?',
     R'{Y}[-:]{m}[-:]{d}(?:[T\x20]{H}:{M}(?::{S})?(?:[Z.][0-9]{{6}})?{z}?)',
     R'{m}/{d}/{Y}(?:{c}{T})?',
@@ -401,6 +408,8 @@ class formats(PatternEnum):
     "Base85 encoded strings"
     a85 = alphabet(R'[!-u]')
     "Ascii85 encoded strings"
+    z85 = alphabet(R'[-0-9a-zA-Z.:+=^!/*?&<>()\[\]{}@%$#]')
+    "Z85 encoded strings"
     b92 = pattern(_pattern_b92)
     "Base92 encoded strings"
     b64any = alphabet(R'(?:[-\w\+/]{4})', postfix=R'(?:(?:[-\w\+/]{2,3})={0,3})?')
@@ -419,7 +428,8 @@ class formats(PatternEnum):
     "Base85 encoded string, separated by whitespace"
     spaced_a85 = alphabet(R'[!-u\s]')
     "Ascii85 encoded string, separated by whitespace"
-
+    spaced_z85 = alphabet(R'[-\s0-9a-zA-Z.:+=^!/*?&<>()\[\]{}@%$#]')
+    "Z85 encoded string, separated by whitespace"
     utf8 = pattern(_pattern_utf8)
     "A sequence of bytes that can be decoded as UTF8."
     hexdump = tokenize(_pattern_hexline, bound='', sep=R'\s*\n')
