@@ -1,5 +1,3 @@
-#!/usr/bin/env python3
-# -*- coding: utf-8 -*-
 """
 Interfaces and classes to read structured data.
 """
@@ -15,11 +13,12 @@ import re
 import struct
 import weakref
 
+from uuid import UUID
+
 from typing import (
     overload,
     cast,
     Any,
-    ByteString,
     Dict,
     Generic,
     Sized,
@@ -35,10 +34,18 @@ from typing import (
 
 if TYPE_CHECKING:
     from collections.abc import Buffer
+    from refinery.lib.types import ByteStr
+
+    T = TypeVar('T', bound=Union[bytearray, bytes, memoryview])
+    B = TypeVar('B', bound=Union[bytearray, bytes, memoryview], default=T)
+    C = TypeVar('C', bound=Union[bytearray, bytes, memoryview])
+    R = TypeVar('R', bound=io.IOBase)
+else:
+    T = TypeVar('T')
+    B = TypeVar('B')
+    C = TypeVar('C')
+    R = TypeVar('R')
 
-T = TypeVar('T', bound=Union[bytearray, bytes, memoryview])
-C = TypeVar('C', bound=Union[bytearray, bytes, memoryview])
-R = TypeVar('R', bound=io.IOBase)
 
 UnpackType = Union[int, bool, float, bytes]
 
@@ -57,7 +64,7 @@ class EOF(EOFError):
     While reading from a `refinery.lib.structures.MemoryFile`, less bytes were available than
     requested. The exception contains the data from the incomplete read.
     """
-    def __init__(self, rest: ByteString = B''):
+    def __init__(self, rest: ByteStr = B''):
         super().__init__('Unexpected end of buffer.')
         self.rest = rest
 
@@ -85,14 +92,13 @@ class StreamDetour(Generic[R]):
         self.stream.seek(self.cursor, io.SEEK_SET)
 
 
-class MemoryFileMethods(Generic[T]):
+class MemoryFileMethods(Generic[T, B]):
     """
     A thin wrapper around (potentially mutable) byte sequences which gives it the features of a
     file-like object.
     """
-    read_as_bytes: bool
-
     _data: T
+    _output: type[B]
     _cursor: int
     _closed: bool
 
@@ -101,24 +107,35 @@ class MemoryFileMethods(Generic[T]):
         END = io.SEEK_END
         SET = io.SEEK_SET
 
+    def __bytes__(self):
+        return bytes(self._data)
+
     def __init__(
         self,
-        data: Optional[Union[bytes, bytearray, memoryview]] = None,
-        read_as_bytes=False,
+        data: Optional[T] = None,
+        output: Optional[type[B]] = None,
         fileno: Optional[int] = None,
         size_limit: Optional[int] = None,
     ) -> None:
         if data is None:
-            data = bytearray()
-        elif size_limit is not None and len(data) > size_limit:
+            if TYPE_CHECKING:
+                data = cast(T, bytearray())
+            else:
+                data = bytearray()
+        if output is None:
+            if TYPE_CHECKING:
+                output = cast(type[B], type(data))
+            else:
+                output = type(data)
+        if size_limit is not None and len(data) > size_limit:
             raise ValueError('Initial data exceeds size limit')
-        self._data = cast(T, data)
+        self._output = output
         self._cursor = 0
         self._closed = False
         self._fileno = fileno
         self._quicksave = 0
-        self.read_as_bytes = read_as_bytes
         self._size_limit = size_limit
+        self._data = data
 
     def close(self) -> None:
         self._closed = True
@@ -173,6 +190,12 @@ class MemoryFileMethods(Generic[T]):
     def detour(self, offset: Optional[int] = None, whence: int = io.SEEK_SET):
         return StreamDetour(cast(io.IOBase, self), offset, whence=whence)
 
+    def detour_absolute(self, offset: Optional[int] = None):
+        return self.detour(offset, io.SEEK_SET)
+
+    def detour_relative(self, offset: Optional[int] = None):
+        return self.detour(offset, io.SEEK_CUR)
+
     def writable(self) -> bool:
         if self._closed:
             return False
@@ -186,15 +209,15 @@ class MemoryFileMethods(Generic[T]):
             out = cast(out)
         return out
 
-    def read(self, size: Optional[int] = None, peek: bool = False) -> Union[bytes, T]:
+    def read(self, size: Optional[int] = None, peek: bool = False) -> B:
         beginning = self._cursor
         if size is None or size < 0:
             end = len(self._data)
         else:
             end = min(self._cursor + size, len(self._data))
         result = self._data[beginning:end]
-        if self.read_as_bytes and not isinstance(result, bytes):
-            result = bytes(result)
+        if not isinstance(result, t := self._output):
+            result = t(result)
         if not peek:
             self._cursor = end
         return result
@@ -214,7 +237,7 @@ class MemoryFileMethods(Generic[T]):
             return mv[cursor:]
         return mv[cursor:cursor + size]
 
-    def read1(self, size: Optional[int] = None, peek: bool = False) -> Union[bytes, T]:
+    def read1(self, size: Optional[int] = None, peek: bool = False) -> B:
         return self.read(size, peek)
 
     def _find_linebreak(self, beginning: int, end: int) -> int:
@@ -224,18 +247,18 @@ class MemoryFileMethods(Generic[T]):
             if self._data[k] == 0xA: return k
         return -1
 
-    def readline(self, size: Optional[int] = None) -> Union[bytes, T]:
+    def readline(self, size: Optional[int] = None) -> B:
         beginning, end = self._cursor, len(self._data)
         if size is not None and size >= 0:
             end = beginning + size
         p = self._find_linebreak(beginning, end)
         self._cursor = end if p < 0 else p + 1
         result = self._data[beginning:self._cursor]
-        if self.read_as_bytes and not isinstance(result, bytes):
-            result = bytes(result)
+        if not isinstance(result, t := self._output):
+            result = t(result)
         return result
 
-    def readlines_iter(self, hint: Optional[int] = None) -> Iterable[Union[bytes, T]]:
+    def readlines_iter(self, hint: Optional[int] = None) -> Iterable[B]:
         if hint is None or hint < 0:
             yield from self
         else:
@@ -245,8 +268,11 @@ class MemoryFileMethods(Generic[T]):
                 total += len(line)
                 yield line
 
-    def readlines(self, hint: Optional[int] = None) -> List[Union[bytes, T]]:
-        return list(self.readlines_iter(hint))
+    def readlines(self, hint: Optional[int] = None) -> list[bytes]:
+        it = self.readlines_iter(hint)
+        if issubclass(self._output, bytes):
+            return list(it)
+        return [bytes(t) for t in it]
 
     def readinto1(self, b) -> int:
         data = self.read(len(b))
@@ -260,6 +286,9 @@ class MemoryFileMethods(Generic[T]):
     def tell(self) -> int:
         return self._cursor
 
+    def skip(self, n: int):
+        self._cursor += n
+
     def seekrel(self, offset: int) -> int:
         return self.seek(offset, io.SEEK_CUR)
 
@@ -388,22 +417,24 @@ class MemoryFileMethods(Generic[T]):
 
     def __getitem__(self, slice):
         result = self._data[slice]
-        if self.read_as_bytes and not isinstance(result, bytes):
-            result = bytes(result)
+        if not isinstance(result, t := self._output):
+            result = t(result)
         return result
 
     def replay(self, offset: int, length: int):
-        if offset not in range(self._cursor + 1):
+        cursor = self._cursor
+        if offset not in range(cursor + 1):
             raise ValueError(F'The supplied delta {offset} is not in the valid range [0,{self._cursor}].')
         rep, r = divmod(length, offset)
-        offset = -offset - len(self) + self._cursor
+        offset = cursor - offset
         replay = self._data[offset:offset + r]
         if rep > 0:
-            replay = bytes(self._data[offset:self._cursor]) * rep + replay
+            # While this is technically a copy, it is faster than repeated calls to write.
+            replay = bytes(self._data[offset:cursor]) * rep + replay
         self.write(replay)
 
 
-class MemoryFile(MemoryFileMethods[T], io.BytesIO):
+class MemoryFile(MemoryFileMethods[T, B], io.BytesIO):
     pass
 
 
@@ -412,7 +443,7 @@ class order(str, enum.Enum):
     little = '<'
 
 
-class StructReader(MemoryFile[T]):
+class StructReader(MemoryFile[T, T]):
     """
     An extension of a `refinery.lib.structures.MemoryFile` which provides methods to read
     structured data.
@@ -452,7 +483,7 @@ class StructReader(MemoryFile[T]):
         self._nbits = 0
         return super().seek(offset, whence)
 
-    def read_exactly(self, size: Optional[int] = None, peek: bool = False) -> Union[bytes, T]:
+    def read_exactly(self, size: Optional[int] = None, peek: bool = False) -> B:
         """
         Read bytes from the underlying stream. Raises a `RuntimeError` when the stream is not currently
         byte-aligned, i.e. when `refinery.lib.structures.StructReader.byte_aligned` is `False`. Raises
@@ -500,8 +531,6 @@ class StructReader(MemoryFile[T]):
         """
         if length is None:
             length = self.remaining_bits
-        if length < 0:
-            raise ValueError
         if bigendian is None:
             bigendian = self.bigendian
         if length < self._nbits:
@@ -592,7 +621,11 @@ class StructReader(MemoryFile[T]):
         for bit in bits:
             yield bool(bit)
 
-    def read_struct(self, spec: str, unwrap=False, peek=False) -> Union[List[UnpackType], UnpackType]:
+    def read_one_struct(self, spec: str, peek=False) -> UnpackType:
+        item, = self.read_struct(spec, peek=peek)
+        return item
+
+    def read_struct(self, spec: str, peek=False) -> List[UnpackType]:
         """
         Read structured data from the stream in any format supported by the `struct` module. The `format`
         argument can be used to override the current byte ordering. If the `unwrap` parameter is `True`, a
@@ -628,8 +661,6 @@ class StructReader(MemoryFile[T]):
             else:
                 part = F'{byteorder}{part}'
                 data.extend(struct.unpack(part, self.read_bytes(struct.calcsize(part))))
-        if unwrap and len(data) == 1:
-            return data[0]
         if peek:
             self.seekset(current_cursor)
         return data
@@ -650,8 +681,17 @@ class StructReader(MemoryFile[T]):
     def i32(self, peek: bool = False) -> int: return signed(self.read_integer(32, peek), 32)
     def i64(self, peek: bool = False) -> int: return signed(self.read_integer(64, peek), 64)
 
-    def f32(self, peek: bool = False) -> float: return cast(float, self.read_struct('f', unwrap=True, peek=peek))
-    def f64(self, peek: bool = False) -> float: return cast(float, self.read_struct('d', unwrap=True, peek=peek))
+    def f32(self, peek: bool = False) -> float: return cast(float, self.read_one_struct('f', peek=peek))
+    def f64(self, peek: bool = False) -> float: return cast(float, self.read_one_struct('d', peek=peek))
+
+    def u8fast(self):
+        try:
+            b = self._data[self._cursor]
+        except IndexError:
+            raise EOFError
+        else:
+            self._cursor += 1
+            return b
 
     def read_byte(self, peek: bool = False) -> int: return self.read_integer(8, peek)
     def read_char(self, peek: bool = False) -> int: return signed(self.read_integer(8, peek), 8)
@@ -689,21 +729,11 @@ class StructReader(MemoryFile[T]):
             self.seekrel(len(terminator))
             return bytearray(data)
 
-    def read_guid(self) -> str:
-        _mode = self.bigendian
-        self.bigendian = False
-        try:
-            a = self.u32()
-            b = self.u16()
-            c = self.u16()
-            d = self.read(2).hex().upper()
-            e = self.read(6).hex().upper()
-        except Exception:
-            raise
-        else:
-            return F'{a:08X}-{b:04X}-{c:04X}-{d}-{e}'
-        finally:
-            self.bigendian = _mode
+    def read_guid(self) -> UUID:
+        return UUID(bytes_le=self.read_bytes(16))
+
+    def read_uuid(self) -> UUID:
+        return UUID(bytes=self.read_bytes(16))
 
     @overload
     def read_c_string(self) -> bytearray:
@@ -752,24 +782,35 @@ class StructReader(MemoryFile[T]):
         ...
 
     @overload
-    def read_length_prefixed(self, *, prefix_size: int = 32, block_size: int = 1) -> Union[T, bytes]:
+    def read_length_prefixed(self, *, prefix_size: int = 32, block_size: int = 1) -> T:
         ...
 
-    def read_length_prefixed(self, prefix_size: int = 32, encoding: Optional[str] = None, block_size: int = 1) -> Union[T, bytes, str]:
+    @overload
+    def read_length_prefixed(self, prefix_size: int, *, block_size: int = 1) -> T:
+        ...
+
+    def read_length_prefixed(self, prefix_size: int = 32, encoding: Optional[str] = None, block_size: int = 1) -> Union[T, str]:
         prefix = self.read_integer(prefix_size) * block_size
         data = self.read(prefix)
         if encoding is not None:
             data = codecs.decode(data, encoding)
         return data
 
-    def read_7bit_encoded_int(self, max_bits: int = 0) -> Optional[int]:
+    def read_7bit_encoded_int(self, max_bits: int = 0, bigendian: bool | None = None) -> int:
         value = 0
-        for shift in itertools.count(0, step=7):
-            b = self.read_byte()
-            value |= (b & 0x7F) << shift
+        shift = 0
+        if bigendian is None:
+            bigendian = self.bigendian
+        while True:
+            b = self.u8fast()
+            if bigendian:
+                value <<= 7
+                value |= (b & 0x7F)
+            else:
+                value |= (b & 0x7F) << shift
             if not b & 0x80:
                 return value
-            if shift > max_bits > 0:
+            if (shift := shift + 7) > max_bits > 0:
                 raise OverflowError('Maximum bits were exceeded by encoded integer.')
 
 
