@@ -1,5 +1,5 @@
-#!/usr/bin/env python3
-# -*- coding: utf-8 -*-
+from __future__ import annotations
+
 from refinery.units import Unit
 
 
@@ -8,7 +8,7 @@ class brotli(Unit):
     Brotli compression and decompression.
     """
 
-    @Unit.Requires('brotlipy', 'all')
+    @Unit.Requires('brotlipy', ['all'])
     def _brotli():
         import brotli
         return brotli
