@@ -1,5 +1,5 @@
-#!/usr/bin/env python3
-# -*- coding: utf-8 -*-
+from __future__ import annotations
+
 from typing import ByteString, NamedTuple, Optional, Dict
 
 from enum import IntFlag
@@ -22,6 +22,7 @@ from .szdd import szdd
 from .zl import zl
 from .qlz import qlz
 from .lzf import lzf
+from .flz import flz
 from .lzw import lzw
 from .nrv import nrv2b, nrv2d, nrv2e
 from .zstd import zstd
@@ -130,6 +131,7 @@ class decompress(Unit):
             bz2,
             zl,
             lzf,
+            flz,
             lzma,
             lzw,
             jcalg,
@@ -143,7 +145,7 @@ class decompress(Unit):
             lznt1,
             nrv2e,
             nrv2d,
-            nrv2b
+            nrv2b,
         ]:
             unit: Unit = engine.assemble()
             _, _, name = unit.name.rpartition('auto-decompress-')
@@ -255,7 +257,7 @@ class decompress(Unit):
 
         for method, engine in self.engines.items():
             self.log_debug(F'attempting engine: {method}')
-            careful = isinstance(engine, (lznt1, lzf, lzjb))
+            careful = isinstance(engine, (lznt1, lzf, flz, lzjb))
             for t in range(self.args.tolerance + 1):
                 if best_current_rating() >= _R.Successful and careful and t > 0:
                     break
@@ -264,6 +266,9 @@ class decompress(Unit):
                 for p in range(0x100):
                     update(decompress(method, engine, 0, p, careful), careful)
 
+        for r, u in best_by_rating.items():
+            self.log_debug(r, u.method)
+
         for r in sorted(best_by_rating, reverse=True):
             if dc := best_by_rating[r]:
                 if not dc.rating & _R.HadOutput:
