@@ -1,5 +1,5 @@
-#!/usr/bin/env python3
-# -*- coding: utf-8 -*-
+from __future__ import annotations
+
 from enum import IntEnum, IntFlag
 from typing import ByteString, Generator, Optional
 from zlib import adler32, crc32
@@ -276,7 +276,7 @@ class lzo(Unit):
             )
 
     @classmethod
-    def handles(self, data: bytearray) -> Optional[bool]:
+    def handles(cls, data: bytearray) -> Optional[bool]:
         sig = LZO.SIGNATURE
         if data[:len(sig)] == sig:
             return True
