@@ -1,5 +1,5 @@
-#!/usr/bin/env python3
-# -*- coding: utf-8 -*-
+from __future__ import annotations
+
 from enum import IntEnum
 from typing import Optional
 from array import array
@@ -136,7 +136,7 @@ class lzw(Unit):
         return out.getvalue()
 
     @classmethod
-    def handles(self, data: bytearray) -> Optional[bool]:
-        sig = self._MAGIC
+    def handles(cls, data: bytearray) -> Optional[bool]:
+        sig = cls._MAGIC
         if data[:len(sig)] == sig:
             return True
