@@ -1,5 +1,5 @@
-#!/usr/bin/env python3
-# -*- coding: utf-8 -*-
+from __future__ import annotations
+
 import abc
 
 from refinery.lib.structures import StructReader, MemoryFile
