@@ -1,5 +1,3 @@
-#!/usr/bin/env python3
-# -*- coding: utf-8 -*-
 from __future__ import annotations
 
 from refinery.units import Unit, RefineryPartialResult
@@ -67,7 +65,7 @@ class pkw(Unit):
         return output.getvalue()
 
     @classmethod
-    def handles(self, data: bytearray) -> bool:
+    def handles(cls, data: bytearray) -> bool:
         return (len(data) > 2) and (0 <= data[0] <= 1) and (4 <= data[1] <= 6)
 
 
