@@ -1,5 +1,5 @@
-#!/usr/bin/env python3
-# -*- coding: utf-8 -*-
+from __future__ import annotations
+
 from refinery.lib.maru import maru32digest
 from refinery.units.crypto.hash import HashUnit, Arg
 
@@ -16,5 +16,5 @@ class maru(HashUnit):
     ):
         super().__init__(seed=seed, text=text, reps=reps)
 
-    def _algorithm(self, data: bytes) -> bytes:
+    def _algorithm(self, data) -> bytes:
         return maru32digest(data, self.args.seed)
