@@ -1,8 +1,9 @@
-#!/usr/bin/env python3
-# -*- coding: utf-8 -*-
 """
 Implements password hashing algorithms.
 """
+from __future__ import annotations
+
+import codecs
 from refinery.units.crypto.hash import HashUnit
 
 
@@ -10,6 +11,6 @@ class ntlm(HashUnit):
     """
     Returns the Windows NTLM hash of the input.
     """
-    def _algorithm(self, data: bytes) -> bytes:
+    def _algorithm(self, data) -> bytes:
         from Cryptodome.Hash import MD4
-        return MD4.new(data.decode(self.codec).encode('utf-16le')).digest()
+        return MD4.new(codecs.decode(data, self.codec).encode('utf-16le')).digest()
