@@ -1,5 +1,5 @@
-#!/usr/bin/env python3
-# -*- coding: utf-8 -*-
+from __future__ import annotations
+
 import base64
 import re
 
@@ -19,6 +19,6 @@ class a85(Unit):
         return base64.a85decode(data)
 
     @classmethod
-    def handles(self, data: bytearray):
+    def handles(cls, data: bytearray):
         from refinery.lib.patterns import formats
-        return formats.spaced_a85.value.fullmatch(data)
+        return formats.spaced_a85.value.bin.fullmatch(data) is not None
