@@ -1,5 +1,5 @@
-#!/usr/bin/env python3
-# -*- coding: utf-8 -*-
+from __future__ import annotations
+
 from refinery.units.encoding.base import base
 
 
@@ -15,7 +15,7 @@ class b58(base):
     def handles(cls, data):
         from refinery.lib.patterns import formats
         return (
-            formats.b58.value.fullmatch(data)
-            and not formats.hex.value.fullmatch(data)
-            and not formats.b32.value.fullmatch(data)
+            formats.b58.value.bin.fullmatch(data)
+            and not formats.hex.value.bin.fullmatch(data)
+            and not formats.b32.value.bin.fullmatch(data)
         )
