@@ -1,5 +1,3 @@
-#!/usr/bin/env python3
-# -*- coding: utf-8 -*-
 from __future__ import annotations
 from typing import TYPE_CHECKING
 
@@ -27,7 +25,7 @@ class xt7z(ArchiveUnit, docs='{0}{s}{PathExtractorUnit}'):
     """
     Extract files from a 7zip archive.
     """
-    @ArchiveUnit.Requires('py7zr', 'arc', 'default', 'extended')
+    @ArchiveUnit.Requires('py7zr', ['arc', 'default', 'extended'])
     def _py7zr():
         import py7zr
         import py7zr.exceptions
