@@ -1,5 +1,3 @@
-#!/usr/bin/env python3
-# -*- coding: utf-8 -*-
 from __future__ import annotations
 from typing import Iterable, Optional
 
@@ -14,7 +12,7 @@ class xtnuitka(PathExtractorUnit):
     """
     _MAGIC = B'KA'
 
-    @PathExtractorUnit.Requires('pyzstd', 'arc')
+    @PathExtractorUnit.Requires('pyzstd', ['arc'])
     def _pyzstd():
         import pyzstd
         return pyzstd
