@@ -1,5 +1,3 @@
-#!/usr/bin/env python3
-# -*- coding: utf-8 -*-
 from __future__ import annotations
 from typing import TYPE_CHECKING, NamedTuple
 
@@ -12,22 +10,21 @@ import zlib
 import os
 import os.path
 import contextlib
+import codecs
 import dataclasses
-import sys
-
-from importlib.util import MAGIC_NUMBER
 
 from refinery.units.formats.archive import Arg, ArchiveUnit
 from refinery.units.pattern.carve import carve
+from refinery.units.formats.pym import Marshal
 from refinery.lib.structures import MemoryFile, StreamDetour, Struct, StructReader
 from refinery.lib.tools import NoLogging, normalize_word_separators
+from refinery.lib.shared import xdis, decompyle3, uncompyle6
 
 from Cryptodome.Cipher import AES
 
-
 if TYPE_CHECKING:
     from types import CodeType
-    from typing import Callable, Dict, List, Tuple, Optional, Set, Union, Generator, Iterable
+    from typing import cast, Callable, Dict, List, Tuple, Optional, Set, Union, Generator, Iterable
     from xdis import Instruction
     from refinery.lib.types import ByteStr
 
@@ -39,7 +36,9 @@ class Unmarshal(enum.IntEnum):
 
 
 def version2tuple(version: str):
-    return tuple(int(k, 10) for k in re.fullmatch(R'^(\d+\.\d+(?:\.\d+)?)(.*)$', version).group(1).split('.'))
+    if m := re.fullmatch(R'^(\d+\.\d+(?:\.\d+)?)(.*)$', version):
+        return tuple(int(k, 10) for k in m.group(1).split('.'))
+    raise ValueError(version)
 
 
 def decompress_peek(buffer, size=512) -> Optional[bytes]:
@@ -61,7 +60,7 @@ class Code(NamedTuple):
 def extract_code_from_buffer(buffer: ByteStr, file_name: Optional[str] = None) -> Generator[Code, None, None]:
     code_objects = {}
     file_name = file_name or '<unknown>'
-    load = xtpyi._xdis.load.load_module_from_file_object
+    load = xdis.load.load_module_from_file_object
     with NoLogging(NoLogging.Mode.STD_ERR):
         version, timestamp, magic_int, codes, is_pypy, _, _ = load(MemoryFile(buffer), file_name, code_objects)
     if not isinstance(codes, list):
@@ -71,37 +70,36 @@ def extract_code_from_buffer(buffer: ByteStr, file_name: Optional[str] = None) -
 
 
 def disassemble_code(code: CodeType, version=None) -> Iterable[Instruction]:
-    dis = xtpyi._xdis
     opc = None
     if version is not None:
         if isinstance(version, float):
             version = str(version)
         if not isinstance(version, str):
-            version = dis.version_info.version_tuple_to_str(version)
+            version = xdis.version_info.version_tuple_to_str(version)
         with contextlib.suppress(KeyError):
-            opc = dis.op_imports.op_imports[version]
-    return dis.std.Bytecode(code, opc=opc)
+            opc = xdis.op_imports.op_imports[version]
+    return xdis.std.Bytecode(code, opc=opc)
 
 
 def decompile_buffer(buffer: Union[Code, ByteStr], file_name: Optional[str] = None) -> ByteStr:
-    main: xtpyi = xtpyi
     errors = ''
     python = ''
-    codes = [buffer]
 
     if not isinstance(buffer, Code):
         codes = list(extract_code_from_buffer(buffer, file_name))
+    else:
+        codes = [buffer]
 
     def _engines():
         nonlocal errors
         try:
-            dc = main._decompyle3
+            dc = decompyle3.main.decompile
         except ImportError:
             errors += '# The decompiler decompyle3 is not installed.\n'
         else:
             yield 'decompyle3', dc
         try:
-            dc = main._uncompyle6
+            dc = uncompyle6.main.decompile
         except ImportError:
             errors += '# The decompiler decompyle3 is not installed.\n'
         else:
@@ -113,10 +111,10 @@ def decompile_buffer(buffer: Union[Code, ByteStr], file_name: Optional[str] = No
         errors += '# (all missing, install one of the above to enable decompilation)'
 
     for code in codes:
-        for name, engine in engines.items():
+        for name, decompile in engines.items():
             with io.StringIO(newline='') as output, NoLogging(NoLogging.Mode.ALL):
                 try:
-                    engine.main.decompile(
+                    decompile(
                         co=code.container,
                         bytecode_version=code.version,
                         out=output,
@@ -140,13 +138,13 @@ def decompile_buffer(buffer: Union[Code, ByteStr], file_name: Optional[str] = No
             python.pop()
         python.reverse()
         python = ''.join(python)
-        return python.encode(main.codec)
+        return python.encode('utf8')
     if not isinstance(buffer, Code):
         embedded = bytes(buffer | carve('printable', single=True))
         if len(buffer) - len(embedded) < 0x20:
             return embedded
     disassembly = MemoryFile()
-    with io.TextIOWrapper(disassembly, main.codec, newline='\n') as output:
+    with io.TextIOWrapper(disassembly, 'utf8', newline='\n') as output:
         output.write(errors)
         output.write('# Generating Disassembly:\n\n')
         for code in codes:
@@ -206,7 +204,7 @@ def make_decompiled_item(name: str, data: ByteStr, *magics) -> PiMeta:
             except Exception as exception:
                 error = exception
         return '\n'.join(F'# {line}'
-            for line in str(error).splitlines(True)).encode(xtpyi.codec)
+            for line in str(error).splitlines(True)).encode('utf8')
 
     return PiMeta(PiType.DECOMPILED, F'{name}.py', extract)
 
@@ -223,9 +221,6 @@ class PYZ(Struct):
             raise ValueError('invalid magic')
         magic = bytes(reader.read(4))
         with contextlib.suppress(KeyError, AttributeError):
-            xdis = xtpyi._xdis
-            if isinstance(xdis, property):
-                xdis = xdis.fget()
             version = xdis.magics.versions[magic]
         vtuple = version2tuple(version)
         padding_size = 4
@@ -244,24 +239,8 @@ class PYZ(Struct):
             toc_data = self.reader.read()
         try:
             toc = marshal.loads(toc_data)
-        except Exception as error:
-            if MAGIC_NUMBER != self.magic[:4]:
-                xdis = xtpyi._xdis
-                if isinstance(xdis, property):
-                    xdis = xdis.fget()
-                _ord = xdis.marsh.Ord
-                xdis.marsh.Ord = ord  # monkey-patch workaround for bug in xdis
-                try:
-                    toc = xdis.marsh.load(
-                        MemoryFile(self.data), self.version)
-                except Exception:
-                    pass
-                else:
-                    error = None
-                finally:
-                    xdis.marsh.Ord = _ord
-            if error is not None:
-                raise error
+        except Exception:
+            toc = Marshal(memoryview(toc_data)).object()
 
         if isinstance(toc, list):
             try:
@@ -269,19 +248,19 @@ class PYZ(Struct):
             except Exception as error:
                 self.entries = []
                 self.error = error
-                return
+                return False
+
+        if TYPE_CHECKING:
+            toc = cast(dict[str | bytes, tuple[int, int, int]], toc)
 
         failures = 0
         attempts = len(toc)
 
-        for name, (pzt, offset, length) in toc.items():
+        for name, (_pzt, offset, length) in toc.items():
+            if not isinstance(name, str):
+                name = codecs.decode(name, 'utf-8')
             try:
-                name: str
-                name = name.decode('utf-8')
-            except AttributeError:
-                pass
-            try:
-                pzt = PzType(pzt)
+                pzt = PzType(_pzt)
             except Exception:
                 pzt = PzType.DATA
 
@@ -346,10 +325,10 @@ class PiTOCEntry(Struct):
         try:
             name = name.decode('utf8', 'backslashreplace')
         except Exception:
-            name = None
-        if not all(part.isprintable() for part in re.split('\\s*', name)):
-            raise RuntimeError('Refusing to process TOC entry with non-printable name.')
-        name = name or str(uuid.uuid4())
+            name = str(uuid.uuid4())
+        else:
+            if not all(part.isprintable() for part in re.split('\\s*', name)):
+                raise RuntimeError('Refusing to process TOC entry with non-printable name.')
         if entry_type == B'Z':
             entry_type = B'z'
         try:
@@ -465,21 +444,22 @@ class PyInstallerArchiveEpilogue(Struct):
             self.files[extracted.name] = extracted
             is_crypto_key = name.endswith('crypto_key')
 
-            if len(magics) == 1 and data[:4] != magics[0]:
+            if len(magics) == 1 and data[:4] != magics[0][:4]:
                 extracted.data = magics[0] + data
 
             if is_crypto_key or self.decompile:
                 decompiled = make_decompiled_item(name, data, *magics)
+
                 if entry.type is PiType.SOURCE:
                     decompiled.type = PiType.USERCODE
                 self.files[decompiled.name] = decompiled
 
-            if is_crypto_key:
-                for key in decompiled.unpack() | carve('string', decode=True):
-                    if len(key) != 0x10:
-                        continue
-                    xtpyi.logger.info(F'found key: {key.decode(xtpyi.codec)}')
-                    keys.add(key)
+                if is_crypto_key:
+                    for key in decompiled.unpack() | carve('string', decode=True):
+                        if len(key) != 0x10:
+                            continue
+                        xtpyi.logger.info(F'found key: {key.decode(xtpyi.codec)}')
+                        keys.add(key)
 
         if unmarshal is Unmarshal.No:
             return
@@ -546,44 +526,6 @@ class xtpyi(ArchiveUnit, docs='{0}{s}{PathExtractorUnit}'):
             user_code=user_code,
         )
 
-    @ArchiveUnit.Requires('xdis', 'arc', 'python', 'extended')
-    def _xdis():
-        import xdis.load
-        import xdis.magics
-        import xdis.marsh
-        import xdis.op_imports
-        import xdis.version_info
-        import xdis
-        A, B, C, *_ = sys.version_info
-        version = F'{A}.{B}.{C}'
-        canonic = F'{A}.{B}'
-        if version not in xdis.magics.canonic_python_version:
-            import importlib.util
-            magic = importlib.util.MAGIC_NUMBER
-            xdis.magics.add_magic_from_int(xdis.magics.magic2int(magic), version)
-            xdis.magics.by_magic.setdefault(magic, set()).add(version)
-            xdis.magics.by_version[version] = magic
-            xdis.magics.magics[canonic] = magic
-            xdis.magics.canonic_python_version[canonic] = canonic
-            xdis.magics.add_canonic_versions(version, canonic)
-            xdis.op_imports.op_imports.setdefault(canonic,
-                next(iter(reversed(xdis.op_imports.op_imports.values()))))
-        del A, B, C, version
-        import xdis.std
-        return xdis
-
-    @ArchiveUnit.Requires('uncompyle6', 'arc', 'python', 'extended')
-    def _uncompyle6():
-        import uncompyle6
-        import uncompyle6.main
-        return uncompyle6
-
-    @ArchiveUnit.Requires('decompyle3', 'arc', 'python')
-    def _decompyle3():
-        import decompyle3
-        import decompyle3.main
-        return decompyle3
-
     def unpack(self, data):
         view = memoryview(data)
         positions = [m.start() for m in re.finditer(re.escape(PyInstallerArchiveEpilogue.MagicSignature), view)]
