@@ -1,5 +1,3 @@
-#!/usr/bin/env python3
-# -*- coding: utf-8 -*-
 from __future__ import annotations
 from typing import TYPE_CHECKING, Container
 from collections import deque
@@ -21,7 +19,7 @@ class vmemref(Unit):
     at the given offset.
     """
 
-    @Unit.Requires('smda<2.0', 'all')
+    @Unit.Requires('smda<2.0', ['all'])
     def _smda():
         import datetime
         datetime.UTC = datetime.timezone.utc
@@ -140,7 +138,7 @@ class vmemref(Unit):
         avoid = set()
 
         for symbol in exe.symbols():
-            if not symbol.code:
+            if not symbol.function:
                 continue
             avoid.add(exe.location_from_address(symbol.address).virtual.box)
 
