@@ -1,5 +1,3 @@
-#!/usr/bin/env python3
-# -*- coding: utf-8 -*-
 from __future__ import annotations
 
 if True:
@@ -8,7 +6,7 @@ if True:
     FG = colorama.Fore
     RS = colorama.Style.RESET_ALL
 
-from typing import Any, Union, List, Dict, TYPE_CHECKING
+from typing import cast, Any, Union, List, Dict, TYPE_CHECKING
 
 import enum
 import functools
@@ -18,12 +16,20 @@ from refinery.units import Arg, Unit
 from refinery.lib.executable import Arch, Range
 from refinery.lib.types import bounds, INF
 from refinery.lib.meta import metavars
-from refinery.lib.tools import isbuffer, exception_to_string, NoLogging
-from refinery.lib.emulator import Emulator, SpeakeasyEmulator, UnicornEmulator, IcicleEmulator, Hook, EmulationError
+from refinery.lib.tools import isbuffer, exception_to_string
 from refinery.lib.intervals import MemoryIntervalUnion
 from refinery.lib.argformats import PythonExpression, ParserVariableMissing
 from refinery.lib.structures import StructReader
 
+from refinery.lib.emulator import (
+    Emulator,
+    SpeakeasyEmulator,
+    UnicornEmulator,
+    IcicleEmulator,
+    Hook,
+    EmulationError,
+)
+
 from dataclasses import dataclass, field
 from collections import defaultdict
 
@@ -219,7 +225,7 @@ class VStackEmulatorMixin(Emulator[Any, Any, EmuState]):
             not state.cfg.log_stack_cookies
             and self.sp ^ unsigned_value == state.last_read
         ):
-            skipped = 'stack cookie'
+            skipped = 'no -E and stack cookie written'
         elif size not in bounds[state.cfg.write_range]:
             skipped = 'size excluded'
         elif (
@@ -227,13 +233,13 @@ class VStackEmulatorMixin(Emulator[Any, Any, EmuState]):
             and not state.cfg.log_writes_in_calls
             and address in range(state.callstack_ceiling - 0x200, state.callstack_ceiling)
         ):
-            skipped = 'inside call'
+            skipped = 'no -W and inside call'
         elif not state.cfg.log_stack_addresses and unsigned_value in self.stackrange():
-            skipped = 'stack address'
+            skipped = 'no -X and stack address written'
         elif not state.cfg.log_other_addresses and not self.exe.blob:
             for s in self.exe.sections():
                 if address in s.virtual:
-                    skipped = F'write to section {s.name}'
+                    skipped = F'no -Y and write to section {s.name}'
                     break
 
         if (
@@ -244,7 +250,7 @@ class VStackEmulatorMixin(Emulator[Any, Any, EmuState]):
         ):
             try:
                 if any(self.mem_read(address, size)):
-                    skipped = 'zero overwrite'
+                    skipped = 'no -Z and zero overwrite detected'
             except Exception:
                 pass
 
@@ -270,6 +276,9 @@ class VStackEmulatorMixin(Emulator[Any, Any, EmuState]):
         if address == self.state.last_api:
             self.state.last_api = None
             return True
+        if address == (1 << self.exe.pointer_size) - 1:
+            self.halt()
+            return False
         msg = F'{state.fmt(address)}:{size:02X} memory error'
         if self.is_mapped(address, size):
             vstack.log_info(self.state.log(F'{msg}; fatal, this area was already mapped'))
@@ -296,17 +305,21 @@ class VStackEmulatorMixin(Emulator[Any, Any, EmuState]):
                 self.set_register(reg, tos)
             _init.clear()
 
-        state.ticks -= 1
-        state.visits[address] += 1
-        if state.visits[address] > state.cfg.max_visits > 0:
-            vstack.log_info(
-                F'aborting emulation: 0x{address:0{self.exe.pointer_size // 8}X}'
-                F' was visited more than {state.cfg.max_visits} times.')
-            self.halt()
-            return False
-        if address == state.stop or state.ticks == 0:
+        if (max_visits := state.cfg.max_visits) > 0:
+            state.visits[address] += 1
+            if state.visits[address] > max_visits:
+                vstack.log_info(
+                    F'aborting emulation: 0x{address:0{self.exe.pointer_size // 8}X}'
+                    F' was visited more than {state.cfg.max_visits} times.')
+                self.halt()
+                return False
+
+        if address == state.stop or (ticks := state.ticks - 1) <= 0:
             self.halt()
             return False
+        else:
+            state.ticks = ticks
+
         waiting = state.waiting
         callstack = state.callstack
         depth = len(callstack)
@@ -318,8 +331,10 @@ class VStackEmulatorMixin(Emulator[Any, Any, EmuState]):
             if retaddr is not None and state.cfg.skip_calls:
                 if state.cfg.skip_calls > 1:
                     self.rv = self.malloc(state.cfg.block_size)
-                self.ip = retaddr
-                self.sp = self.sp + (self.exe.pointer_size // 8)
+                self.ip = _ip = self.pop()
+                if _ip != retaddr:
+                    raise RuntimeError(
+                        'Trying to return from call: top of stack was not the execpted return address.')
                 return
             if depth and address == callstack[-1]:
                 depth -= 1
@@ -368,35 +383,12 @@ class vstack(Unit):
 
     In this pipeline, the eax register is set to `0x2000` before emulation begins.
     """
-    @Unit.Requires('capstone', 'default', 'extended')
-    def _capstone():
-        import capstone
-        return capstone
-
-    @Unit.Requires('unicorn==2.0.1.post1', 'default', 'extended')
-    def _unicorn():
-        import importlib
-        importlib.import_module('setuptools')
-        with NoLogging():
-            import unicorn
-            return unicorn
-
-    @Unit.Requires('speakeasy-emulator-refined', 'extended')
-    def _speakeasy():
-        import speakeasy
-        return speakeasy
-
-    @Unit.Requires('icicle-emu', 'all')
-    def _icicle():
-        import icicle
-        return icicle
-
     def __init__(
         self,
         *address: Arg.NumSeq(metavar='start', help='Specify the (virtual) addresses of a stack string instruction sequences.'),
         stop: Arg.Number('-s', metavar='stop', help='Optional: Stop when reaching this address.') = None,
         base: Arg.Number('-b', metavar='Addr', help='Optionally specify a custom base address B.') = None,
-        arch: Arg.Option('-a', help='Specify for blob inputs: {choices}', choices=Arch) = Arch.X32,
+        arch: Arg.Option('-a', metavar='Arch', help='Specify for blob inputs: {choices}', choices=Arch) = Arch.X32,
         engine: Arg.Option('-e', group='EMU', choices=_engine, metavar='E',
             help='The emulator engine. The default is {default}, options are: {choices}') = _engine.unicorn,
         se: Arg.Switch(group='EMU', help='Equivalent to --engine=speakeasy') = False,
@@ -469,12 +461,14 @@ class vstack(Unit):
         engine: _engine = args.engine
         flags = Hook.Default
         self.log_debug(F'attempting to use {engine.name}')
-        getattr(self, F'_{engine.name}')
 
         if engine is _engine.speakeasy:
             flags |= Hook.ApiCall
 
-        class Emu(engine.value, VStackEmulatorMixin):
+        base: type[Emulator] = cast('type[Emulator]', engine.value) \
+            if TYPE_CHECKING else engine.value
+
+        class Emu(base, VStackEmulatorMixin):
             pass
 
         emu = Emu(
@@ -528,8 +522,8 @@ class vstack(Unit):
             for filter in [
                 lambda s: s.get_name().casefold() == a.casefold(),
                 lambda s: s.name == a,
-                lambda s: s.code,
-                lambda s: s.exported
+                lambda s: s.function,
+                lambda s: s.exported,
             ]:
                 symbols = [s for s in symbols if filter(s)]
                 if len(symbols) == 1:
@@ -550,6 +544,7 @@ class vstack(Unit):
         for cursor in addresses:
             state = EmuState(cfg, cursor, emu.exe.pointer_size // 4, stop=args.stop)
             emu.reset(state)
+            emu.push((1 << emu.exe.pointer_size) - 1)
 
             for reg in emu.general_purpose_registers():
                 if reg not in register_values:
