@@ -1,8 +1,8 @@
-#!/usr/bin/env python3
-# -*- coding: utf-8 -*-
 """
 Java disassembler. The main logic is implemented int `refinery.lib.java.JvOpCode`.
 """
+from __future__ import annotations
+
 import re
 import io
 import collections
@@ -211,5 +211,5 @@ class jvdasm(PathExtractorUnit):
                 yield UnpackResult(path, display.getvalue().encode(self.codec))
 
     @classmethod
-    def handles(self, data):
+    def handles(cls, data):
         return data[:4] == B'\xCA\xFE\xBA\xBE'
