@@ -1,5 +1,5 @@
-#!/usr/bin/env python3
-# -*- coding: utf-8 -*-
+from __future__ import annotations
+
 from refinery.lib import xml
 from refinery.units.formats import PathExtractorUnit, UnpackResult
 from refinery.units.formats.office.xtdoc import xtdoc
@@ -9,7 +9,7 @@ class docmeta(PathExtractorUnit):
     """
     Extract metadata from Word Documents such as custom document properties.
     """
-    @PathExtractorUnit.Requires('olefile', 'formats', 'office')
+    @PathExtractorUnit.Requires('olefile', ['formats', 'office'])
     def _olefile():
         import olefile
         return olefile
