@@ -1,5 +1,5 @@
-#!/usr/bin/env python3
-# -*- coding: utf-8 -*-
+from __future__ import annotations
+
 from typing import Optional
 
 from refinery.units.formats import PathExtractorUnit, UnpackResult
@@ -25,7 +25,7 @@ class xtdoc(PathExtractorUnit):
     Extract files from an OLE document such as a Microsoft Word DOCX file.
     """
 
-    @PathExtractorUnit.Requires('olefile', 'formats', 'office', 'extended')
+    @PathExtractorUnit.Requires('olefile', ['formats', 'office', 'extended'])
     def _olefile():
         import olefile
         return olefile
