@@ -1,5 +1,5 @@
-#!/usr/bin/env python3
-# -*- coding: utf-8 -*-
+from __future__ import annotations
+
 from uuid import uuid4
 
 from refinery.lib.tools import NoLogging
@@ -10,7 +10,7 @@ class xtvba(PathExtractorUnit):
     """
     Extract VBA macro code from Office documents.
     """
-    @PathExtractorUnit.Requires('oletools', 'formats', 'office', 'extended')
+    @PathExtractorUnit.Requires('oletools', ['formats', 'office', 'extended'])
     def _olevba():
         with NoLogging(NoLogging.Mode.ALL):
             import oletools.olevba
