@@ -1,5 +1,3 @@
-#!/usr/bin/env python3
-# -*- coding: utf-8 -*-
 from __future__ import annotations
 
 import contextlib
@@ -32,7 +30,7 @@ class winreg(PathExtractorUnit):
     """
     Extract values from a Windows registry hive or from a registry export (.reg file).
     """
-    @PathExtractorUnit.Requires('python-registry', 'formats')
+    @PathExtractorUnit.Requires('python-registry', ['formats'])
     def _registry():
         import Registry
         import Registry.Registry
