@@ -1,5 +1,5 @@
-#!/usr/bin/env python3
-# -*- coding: utf-8 -*-
+from __future__ import annotations
+
 from refinery.lib.structures import MemoryFile
 from refinery.lib.meta import metavars, is_valid_variable_name
 from refinery.lib import xml
@@ -40,5 +40,5 @@ class xtxml(XMLToPathExtractorUnit):
         yield from walk(root, path(root))
 
     @classmethod
-    def handles(self, data):
+    def handles(cls, data):
         return xml.is_xml(data)
