@@ -1,5 +1,5 @@
-#!/usr/bin/env python3
-# -*- coding: utf-8 -*-
+from __future__ import annotations
+
 from typing import Generator, Optional
 
 import sys
@@ -63,7 +63,7 @@ class peek(HexViewer):
             stdout=stdout,
         )
 
-    @HexViewer.Requires('colorama', 'display', 'default', 'extended')
+    @HexViewer.Requires('colorama', ['display', 'default', 'extended'])
     def _colorama():
         import colorama
         return colorama
